﻿using System;
using System.Diagnostics;
using System.Threading;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;
using Windows.Foundation;
using Windows.UI.Xaml.Controls;

namespace TestApplication
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// I2C 장치
        /// </summary>
        private I2cDevice i2cDevice;

        /// <summary>
        /// 타이머
        /// </summary>
        private Timer timer;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            InitializeADT7410();

            Unloaded += Page_Unloaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 페이지 언로드시 처리하기 - Page_Unloaded(sender, e)

        /// <summary>
        /// 페이지 언로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Page_Unloaded(object sender, object e)
        {
            if(this.i2cDevice != null)
            {
                this.i2cDevice.Dispose();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 온도 계산하기 - CalculateTemperature(highValue, lowValue)
 
        /// <summary>
        /// 온도 계산하기
        /// </summary>
        /// <param name="highValue">상위 값</param>
        /// <param name="lowValue">하위 값</param>
        /// <returns>온도</returns>
        private static double CalculateTemperature(int highValue, int lowValue)
        {
            int value = (highValue << 5) + (lowValue >> 3);

            if(value > 4096) // 온도가 영하인 경우
            {
                value = -1 * (8192 - value);
            }

            double temperature = value * 0.0625;

            return temperature;
        }

        #endregion
        #region 타이머 콜백 처리하기 - TimerCallback(state)

        /// <summary>
        /// 타이머 콜백 처리하기
        /// </summary>
        /// <param name="state">상태</param>
        private void TimerCallback(object state)
        {
            string temperatureString;
 
            byte[] byteArray = new byte[2];

            try
            {
                this.i2cDevice.Read(byteArray);
            }
            catch(Exception exception)
            {
                Debug.WriteLine("예외 : " + exception.Message);
            }

            int highValue = ((int)byteArray[0]);
            int lowValue  = ((int)byteArray[1]);

            double temperature = CalculateTemperature(highValue, lowValue);

            Debug.WriteLine(temperature);

            temperatureString = String.Format("온도 : {0:F4} ℃", temperature);

            IAsyncAction task = this.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                this.textBlock.Text = temperatureString;
            });
        }

        #endregion
        #region ADT7410 초기화하기 - InitializeADT7410()

        /// <summary>
        /// ADT7410 초기화하기
        /// </summary>
        private async void InitializeADT7410()
        {
            try
            {
                string advancedQuerySyntax = I2cDevice.GetDeviceSelector();

                DeviceInformationCollection deviceInformationCollection = await DeviceInformation.FindAllAsync(advancedQuerySyntax);

                I2cConnectionSettings i2cConnectionSettings = new I2cConnectionSettings(0x48);

                i2cConnectionSettings.BusSpeed = I2cBusSpeed.FastMode;

                this.i2cDevice = await I2cDevice.FromIdAsync(deviceInformationCollection[0].Id, i2cConnectionSettings);
            }
            catch(Exception exception)
            {
                Debug.WriteLine("예외 : " + exception.Message);
            }

            this.timer = new Timer(TimerCallback, null, 0, 1000);
        }

        #endregion
    }
}